// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_IOSTREAM
#include <iostream>
#else
#include <stdio.h>
#endif

#include "Rectangle.hh"
#include "BoundingBox.hh"

void
BoundingBox::Zero()
{
  zero   = true;
  ascent = descent = 0;
  width  = 0;
}

void
BoundingBox::Append(const BoundingBox& box)
{  
  width += box.width;

  if (zero) {
    ascent  = box.ascent;
    descent = box.descent;
    zero = false;
  } else {
    ascent  = scaledMax(ascent, box.ascent);
    descent = scaledMax(descent, box.descent);
  }
}

void
BoundingBox::Append(scaled w)
{
  width += w;
}

void
BoundingBox::ToRectangle(scaled x, scaled y, Rectangle& rect) const
{
  rect.x = x;
  rect.y = y - ascent;
  rect.width = width;
  rect.height = GetHeight();
}

void
BoundingBox::Dump() const
{
#if HAVE_IOSTREAM
  if (IsZero()) cout << "[null box]";
  else cout << '[' << sp2ipx(width) << ",+" << sp2ipx(ascent) << ",-" << sp2ipx(descent) << ']';
#else
  if (IsZero()) printf("[null box]");
  else printf("[%d,+%d,-%d]", sp2ipx(width), sp2ipx(ascent), sp2ipx(descent));
#endif
}

#ifdef HAVE_IOSTREAM
ostream&
operator<<(ostream& os, const BoundingBox& box)
{
  box.Dump();
  return os;
}
#endif
