// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>

#include "Font.hh"
#include "UnitValue.hh"

Font::Font(NativeFontPtr native_font)
{
  assert(native_font != NULL);

  font = native_font;
  gdk_font_ref(font);
}

Font::Font(const Font& f)
{
  font = f.font;
  gdk_font_ref(font);
}

Font::~Font()
{
  gdk_font_unref(font);
}

scaled
Font::GetAscent() const
{
  return px2sp(font->ascent);
}

scaled
Font::GetDescent() const
{
  return px2sp(font->descent);
}

scaled
Font::CharWidth(NativeChar ch) const
{
  return px2sp(gdk_char_width_wc(font, ch));
}

scaled
Font::CharHeight(NativeChar ch) const
{
  return px2sp(gdk_char_height(font, ch));
}

scaled
Font::GetEm() const
{
  return CharWidth('M');
}

scaled
Font::GetEx() const
{
  return CharHeight('x');
}

scaled
Font::GetAxis() const
{
  return GetEx() / 2;
}

void
Font::StringBox(const char* s, unsigned length, BoundingBox& box) const
{
  assert(s != NULL);
  assert(font != NULL);

  gint width;
  gint lbearing, rbearing;
  gint ascent, descent;

  gdk_text_extents(font,
		   s, length,
		   &lbearing, &rbearing,
		   &width,
		   &ascent, &descent);

  box.Zero();
  box.width   = px2sp(width);
  box.ascent  = px2sp(ascent);
  box.descent = px2sp(descent);
}

void
Font::StringBox(const NativeChar* s, unsigned length, BoundingBox& box) const
{
  assert(s != NULL);
  assert(font != NULL);

  gint width;
  gint lbearing, rbearing;
  gint ascent, descent;

  gdk_text_extents_wc(font,
		      s, length,
		      &lbearing, &rbearing,
		      &width,
		      &ascent, &descent);

  box.Zero();
  box.width   = px2sp(width);
  box.ascent  = px2sp(ascent);
  box.descent = px2sp(descent);
}

void
Font::CharBox(NativeChar ch, BoundingBox& box) const
{
  StringBox(&ch, 1, box);
}
