// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <string.h>
#ifdef HAVE_IOSTREAM
#include <iostream>
#else
#include <stdio.h>
#endif

#include <gdk/gdk.h>

#include "Font.hh"
#include "globals.hh"
#include "Iterator.hh"
#include "FontManager.hh"

static const char* fontNameTemplate = "-%s-%s-%s-%c-*-*-*-%s-*-*-*-*-%s";

static const Font* SearchGdkFont(const FontAttributes&);

FontManager::FontManager()
{
}

FontManager::~FontManager()
{
}

FontManager::Bucket*
FontManager::SearchFont(const FontAttributes& fa) const
{
  for (Iterator<Bucket*> i(content); i.More(); i.Next()) {
    assert(i() != NULL);
    if (i()->fontAttributes.Equals(fa)) return i();
  }

  return NULL;
}

const Font*
FontManager::GetFont(const FontAttributes& fa)
{
  Bucket* bucket = SearchFont(fa);
  if (bucket != NULL) return bucket->font;

  const Font* font = SearchGdkFont(fa);
  //if (font == NULL) return NULL;

  bucket = new Bucket;
  bucket->fontAttributes = fa;
  bucket->font = font;

  content.Append(bucket);

  return bucket->font;
}

/////////////////////////
// auxiliary functions //
/////////////////////////

static const Font*
SearchGdkFont(const FontAttributes& fa)
{
  const char* foundry = fa.HasFoundry() ? fa.foundry : "*";
  const char* family = fa.HasFamily() ? fa.family : "*";
  char style  = fa.HasStyle() ? ((fa.style == FONT_STYLE_NORMAL) ? 'r' : 'i') : '*';
  const char* weight = fa.HasWeight() ? ((fa.weight == FONT_WEIGHT_NORMAL) ? "medium" : "bold") : "*";
  const char* charSet = fa.HasCharSet() ? fa.charSet : "*-*";
  const char* size;
  if (fa.HasSize()) {
    int ptSize = (fa.size.GetUnitId() == UNIT_PT) ?
      roundFloat(fa.size.GetValue() * 10) :
      roundFloat(sp2pt(fa.size.ToScaledPoints()) * 10);

    char siz[32];

    g_snprintf(siz, 31, "%d", ptSize);
    size = siz;
  } else size = "*";

  static char fontName[128];

  g_snprintf(fontName, 127, fontNameTemplate, foundry, family, weight, style, size, charSet);
  GdkFont* font = gdk_font_load(fontName);

  if (font == NULL && verbose) {
#ifdef HAVE_IOSTREAM
    cout << "WARNING: unable to find font `" << fontName << "'" << endl;
#else
    printf("WARNING: unable to find font `%s'\n", fontName);
#endif
  }

  Font* f = (font != NULL) ? new Font(font) : NULL;

  return f;
}

