// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#ifdef HAVE_IOSTREAM
#include <iostream>
#else
#include <stdio.h>
#endif

#include "Layout.hh"
#include "MathMLOperatorElement.hh"
#include "MathMLEmbellishedOperatorElement.hh"

MathMLEmbellishedOperatorElement::
MathMLEmbellishedOperatorElement(MathMLOperatorElement* op) // the core operator
  : MathMLContainerElement(TAG_MO)
{
  assert(op != NULL);
  coreOp = op;
}

MathMLEmbellishedOperatorElement::~MathMLEmbellishedOperatorElement()
{
}

void
MathMLEmbellishedOperatorElement::Dump() const
{
  assert(content.GetSize() == 1);
  assert(content.GetFirst() != NULL);

  PrintIndentation();
#ifdef HAVE_IOSTREAM
  cout << "<!--embellished operator-->" << endl;
#else
  printf("<!--embellished operator-->\n");
#endif
  IndentMore();
  content.GetFirst()->Dump();
  IndentLess();
  PrintIndentation();
#ifdef HAVE_IOSTREAM
  cout << "<!--/embellished operator-->" << endl;
#else
  printf("<!--/embellished operator-->\n");
#endif
}

void
MathMLEmbellishedOperatorElement::DoBoxedLayout(LayoutId id, BreakId bid, scaled availWidth)
{
  assert(content.GetSize() == 1);
  assert(content.GetFirst() != NULL);
  assert(coreOp != NULL);

  scaled totalPadding = coreOp->GetLeftPadding() + coreOp->GetRightPadding();

  content.GetFirst()->DoBoxedLayout(id, BREAK_NO, scaledMax(0, availWidth - totalPadding));
  box = content.GetFirst()->GetBoundingBox();
  box.width += totalPadding;

  ConfirmLayout(id);
}

#if 0
void
MathMLEmbellishedOperatorElement::DoLayout(LayoutId id, Layout& layout)
{
  assert(content.GetSize() == 1);
  assert(content.GetFirst() != NULL);
  assert(coreOp != NULL);

  layout.Append(coreOp->GetLeftPadding());
  layout.Append(coreOp);
  layout.Append(coreOp->GetRightPadding());
}
#endif

void
MathMLEmbellishedOperatorElement::SetPosition(scaled x, scaled y)
{
  assert(content.GetSize() == 1);
  assert(content.GetFirst() != NULL);
  assert(coreOp != NULL);

  position.x = x;
  position.y = y;

  content.GetFirst()->SetPosition(x + coreOp->GetLeftPadding(), y);
}

#if 0
bool
MathMLEmbellishedOperatorElement::IsBreakable() const
{
  return true;
}
#endif

bool
MathMLEmbellishedOperatorElement::IsEmbellishedOperator() const
{
  return true;
}

BreakId
MathMLEmbellishedOperatorElement::GetBreakability() const
{
  assert(content.GetSize() == 1);
  assert(content.GetFirst() != NULL);
  return content.GetFirst()->GetBreakability();
}
