// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <unicode.h>
#include <wctype.h>

#include "stringAux.hh"
#include "StringUnicode.hh"
#include "allocTextNode.hh"
#include "MathMLCharNode.hh"
#include "MathMLStringNode.hh"

String* allocString(const xmlChar* str)
{
  assert(str != NULL);
  String* sValue = NULL;

  const char* s = reinterpret_cast<const char*>(str);

  if (isPlain(s)) sValue = new StringU<Char8>(s);
  else {
    Uchar*   aValue = u_utf8_to_uc(const_cast<char*>(s));
    // WARNING: from version 0.7 (0.6?) of the Unicode library every converted
    // string has a 2-byte prefix 0xfeff
    unsigned len    = u_strlen(aValue + 1); 
    sValue = new StringU<Char16>(aValue + 1, len);
    free(aValue);
  }

  return sValue;
}

String* allocString(const String& str, unsigned offset, unsigned length)
{
  assert(length >= 1);

  String* sValue = NULL;

  Char big = str.GetBiggestChar(offset, length);
  if      (isPlain(big)) sValue = new StringU<Char8>(str, offset, length);
  else if (isUnicode16(big)) sValue = new StringU<Char16>(str, offset, length);
  else sValue = new StringU<Char32>(str, offset, length);

  return sValue;
}

static String* parsePlainString(const String& s, unsigned& offset)
{
  assert(offset < s.GetLength());

  unsigned start = offset;
  while (offset < s.GetLength() && isPlain(s.GetChar(offset)) && !iswspace(s.GetChar(offset)))
    offset++;

  return allocString(s, start, offset - start);
}

static String* parseNonMarkingChar(const String& s, unsigned& offset)
{
  assert(offset < s.GetLength());
  unsigned start = offset;

  Char ch = s.GetChar(offset++);
#if 0
  bool modifier = (offset < s.GetLength() && s.GetChar(offset) == 0x2063);
  if (modifier) offset++;

  int index = getNonMarkingCharIndex(ch, modifier);
  assert(index >= 0);

  assert(!modifier); // NOT IMPLEMENTED
#endif

  return allocString(s, start, offset - start);
}

static String* parseNonPlainChar(const String& s, unsigned& offset)
{
  assert(offset < s.GetLength());
  assert(!isPlain(s.GetChar(offset)));
  // TODO: some chars are followed by a modifier
  return allocString(s, offset++, 1);
}

MathMLTextNode* parseString(const String& s, unsigned& offset)
{
  assert(offset < s.GetLength());

  const String* sText = NULL;

  Char ch = s.GetChar(offset);
  if (isPlain(ch)) {
    if (iswspace(ch)) sText = parseNonMarkingChar(s, offset);
    else sText = parsePlainString(s, offset);
  } else {
    if (isNonMarkingChar(s, offset)) sText = parseNonMarkingChar(s, offset);
    else sText = parseNonPlainChar(s, offset);
  }
  assert(sText != NULL);

  MathMLTextNode* node = allocTextNode(&sText);

  return node;
}

